/***************************************************************************
                          gpu.c  -  description
                             -------------------
    begin                : Sun Oct 15 2000
    copyright            : (C) 2000 by Pete Bernert
    email                : PeteBernert@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>

#include <GL/gl.h>
#include <GL/glx.h>

#include "gpu.h"

////////////////////////////////////////////////////////////////////////

long (*GPU_init)(void);
long (*GPU_shutdown)(void);
unsigned long (*GPU_open)(unsigned long *,char *,char *);
long (*GPU_close)(void);
char * (*GPU_getLibInfos)(void);
long (*GPU_dmaChain)(unsigned long * baseAddrL, unsigned long addr);
void (*GPU_keypressed)(int keycode);
void (*GPU_updateLace)(void);
unsigned long (*GPU_readStatus)(void);
void (*GPU_writeStatus)(unsigned long gdata);
void (*GPU_writeData)(unsigned long gdata);
void (*GPU_writeDataMem)(unsigned long * pMem, int iSize);
unsigned long (*GPU_readData)(void);
void (*GPU_readDataMem)(unsigned long * pMem, int iSize);
void (*GPU_setMode)(unsigned long gdata);
long (*GPU_getMode)(void);

void     * hGPULib;

////////////////////////////////////////////////////////////////////////

long GPUinit(char * pGpuPlugin)
{
 char * error;

 hGPULib = dlopen (pGpuPlugin, RTLD_NOW);
 if(!hGPULib){fputs(dlerror(),stderr);exit(1);}

 GPU_init=dlsym(hGPULib,"GPUinit");
 if((error=dlerror())!=NULL) {fputs(error, stderr);exit(1);}

 GPU_shutdown=dlsym(hGPULib,"GPUshutdown");
 if((error=dlerror())!=NULL) {fputs(error, stderr);exit(1);}

 GPU_close=dlsym(hGPULib,"GPUclose");
 if((error=dlerror())!=NULL) {fputs(error, stderr);exit(1);}

 GPU_open=dlsym(hGPULib,"GPUopen");
 if((error=dlerror())!=NULL) {fputs(error, stderr);exit(1);}

 GPU_getLibInfos=dlsym(hGPULib,"GPUgetLibInfos");
 if((error=dlerror())!=NULL) {fputs(error, stderr);exit(1);}

 GPU_keypressed=dlsym(hGPULib,"GPUkeypressed");
 if((error=dlerror())!=NULL) {fputs(error, stderr);exit(1);}

 GPU_updateLace=dlsym(hGPULib,"GPUupdateLace");
 if((error=dlerror())!=NULL) {fputs(error, stderr);exit(1);}

 GPU_readStatus=dlsym(hGPULib,"GPUreadStatus");
 if((error=dlerror())!=NULL) {fputs(error, stderr);exit(1);}

 GPU_writeStatus=dlsym(hGPULib,"GPUwriteStatus");
 if((error=dlerror())!=NULL) {fputs(error, stderr);exit(1);}

 GPU_readData=dlsym(hGPULib,"GPUreadData");
 if((error=dlerror())!=NULL) {fputs(error, stderr);exit(1);}

 GPU_writeData=dlsym(hGPULib,"GPUwriteData");
 if((error=dlerror())!=NULL) {fputs(error, stderr);exit(1);}

 GPU_dmaChain=dlsym(hGPULib,"GPUdmaChain");
 if((error=dlerror())!=NULL) {fputs(error, stderr);exit(1);}

 GPU_writeDataMem=dlsym(hGPULib,"GPUwriteDataMem");
 if((error=dlerror())!=NULL) {fputs(error, stderr);exit(1);}

 GPU_readDataMem=dlsym(hGPULib,"GPUreadDataMem");
 if((error=dlerror())!=NULL) {fputs(error, stderr);exit(1);}

 GPU_setMode=dlsym(hGPULib,"GPUsetMode");
 if((error=dlerror())!=NULL) {fputs(error, stderr);exit(1);}

 GPU_getMode=dlsym(hGPULib,"GPUgetMode");
 if((error=dlerror())!=NULL) {fputs(error, stderr);exit(1);}

 return (*GPU_init)();
}

////////////////////////////////////////////////////////////////////////

long GPUshutdown(void)
{
 long lRet=(*GPU_shutdown)();

 dlclose(hGPULib);

 return lRet;
}

////////////////////////////////////////////////////////////////////////

long GPUopen(unsigned long * Disp, char * pCapTxt,char * pCfgFile)

{
 return (*GPU_open)(Disp,pCapTxt,pCfgFile);
}

////////////////////////////////////////////////////////////////////////

long GPUclose(void)
{
 return (*GPU_close)();
}

////////////////////////////////////////////////////////////////////////

void GPUkeypressed(int keycode)
{
 (*GPU_keypressed)(keycode);
}

////////////////////////////////////////////////////////////////////////

void GPUupdateLace(void)
{
 (*GPU_updateLace)();
}

////////////////////////////////////////////////////////////////////////

unsigned long GPUreadStatus(void)
{
 return (*GPU_readStatus)();
}

////////////////////////////////////////////////////////////////////////

void GPUwriteStatus(unsigned long gdata)
{
 (*GPU_writeStatus)(gdata);
}

////////////////////////////////////////////////////////////////////////

unsigned long GPUreadData(void)
{
 return (*GPU_readData)();
}

void GPUreadDataMem(unsigned long * pMem, int iSize)
{
 (*GPU_readDataMem)(pMem,iSize);
}

////////////////////////////////////////////////////////////////////////

void GPUwriteData(unsigned long gdata)
{
 (*GPU_writeData)(gdata);
}

void GPUwriteDataMem(unsigned long * pMem, int iSize)
{
 (*GPU_writeDataMem)(pMem,iSize);
}

////////////////////////////////////////////////////////////////////////

long GPUdmaChain(unsigned long * baseAddrL, unsigned long addr)
{
 return (*GPU_dmaChain)(baseAddrL,addr);
}

////////////////////////////////////////////////////////////////////////

char * GPUgetLibInfos(void)
{
 return (*GPU_getLibInfos)();
}

////////////////////////////////////////////////////////////////////////

void GPUsetMode(unsigned long gdata)
{
 (*GPU_setMode)(gdata);
}

////////////////////////////////////////////////////////////////////////

long GPUgetMode(void)
{
 return (*GPU_getMode)();
}

////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
//
// if you want to do your own gpu plugin:
// just do the above functions in a file called for example
// "gpu.c" and create a dynamical loadable library:
//
// gcc -fPIC -c -Wall gpu.c
// gcc -shared -Wl,-soname,libgpuMySupaDupaStuff.so
//     -o libgpuMySupaDupaStuff.so.1.0.38 gpu.o
//     -lc -lMesaGL -lX11 -lXext -L/usr/X11R6/lib -L/usr/lib
//
////////////////////////////////////////////////////////////////////////


