/***************************************************************************
                          main.c  -  description
                             -------------------
    begin                : Sat Oct 14 15:26:47 CEST 2000
    copyright            : (C) 2000 by Pete Bernert
    email                : PeteBernert@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <GL/glx.h>

#include "gpu.h"

////////////////////////////////////////////////////////////////////////
// globals
////////////////////////////////////////////////////////////////////////

Display  * dspGPU;                                          // needed only for keyboard stuff

////////////////////////////////////////////////////////////////////////
// keyboard handler
////////////////////////////////////////////////////////////////////////

int iCheckKeyboard(void)                                    // KEYBOARD HANDLER
{
 XEvent xE;int iKeyCode,iRet=1;

 while(XPending(dspGPU))                                    // something happened in our window?
  {
   XNextEvent(dspGPU,&xE);                                  // get event
   switch(xE.type)
    {
     case DestroyNotify:                                    // destroy
      iRet=0;
		  break;

     case KeyPress:                                         // key down
      iKeyCode=XLookupKeysym((XKeyEvent *)&xE,0);           // (you may want to store keycodes flags in some table or whatever)
      break;

     case KeyRelease:                                       // key up
      iKeyCode=XLookupKeysym((XKeyEvent *)&xE,0);
      GPUkeypressed(iKeyCode);                              // -> pass keycode to gpu
      if((iKeyCode&0xff)==0x1b) iRet=0;                     // -> ESC is close
      break;
    }
  }

 return iRet;
}

////////////////////////////////////////////////////////////////////////

void InitDemo(void)                                         // INIT DEMO
{
 unsigned long dwInitCmds[]=
  {0xe3000000,0xe403c140,0xe5000000};

 GPUwriteStatus(0x03000000);                                // enable display
 GPUwriteStatus(0x08000021);                                // width 320, interlaced
 GPUwriteStatus(0x0703c000);                                // height 240
 GPUwriteStatus(0x05000000);                                // x,y-pos 0,0
 GPUwriteStatus(0x04000000);                                // no write/read action, do commands
 GPUwriteDataMem(dwInitCmds,3);                             // set drawing area/offset
}

////////////////////////////////////////////////////////////////////////

void PlayDemo(void)                                         // PLAY DEMO
{
 static unsigned long dwCol=0;
 unsigned long dwRect[]=                                    // simple rect
  {0x28000000,0x00500050,0x005000f0,0x00a00050,0x00a000f0};

 dwRect[0]|=dwCol;                                          // add color

 dwCol++;if(dwCol>0x00ffffff) dwCol=0;                      // inc color

 GPUwriteDataMem(dwRect,5);                                 // paint rect

 GPUupdateLace();                                           // emulated vsync
}

////////////////////////////////////////////////////////////////////////
// main
////////////////////////////////////////////////////////////////////////

int main(int argc, char *argv[])                            // MAIN
{
 int i,iRun=1;                                              // init run flag to 1
 char   szDefGpuPlugin[]={"libgpuPeteMesaGL.so"};           // that's the default plugin name
 char * pGpuPlugin=szDefGpuPlugin;                          // init ptr to def name
 char * pCfgFile=NULL;                                      // init cfg file ptr to null
 char * pInfo;

 //---------------------------------------------------------// first check comand line

 for(i=0;i<argc;i++)                                        // loop all args
  {
   if(strstr(argv[i],"-gpu=")==argv[i])                     // -> gpu switch? set ptr
    pGpuPlugin=&argv[i][5];

   if(strstr(argv[i],"-cfg=")==argv[i])                     // -> gpu cfg file? set ptr
    pCfgFile  =&argv[i][5];
  }

 //---------------------------------------------------------// init and open

 if(GPUinit(pGpuPlugin)!=0)                                 // init plugin
  {
   printf("Error GPUinit!");
   return EXIT_SUCCESS;
  }

 printf(GPUgetLibInfos());

 if(GPUopen((unsigned long*)&dspGPU,"Pete's GPU test",      // open gpu window
            pCfgFile)!=0)                                   // NOTE: if you don't need
  {                                                         //  the display pointer for
   printf("Error GPUopen!");                                //  input handers, you can pass
   GPUshutdown();                                           //  NULL as first arg
   return EXIT_SUCCESS;
  }

 //---------------------------------------------------------// main loop

 InitDemo();

 while(iRun)
  {
   PlayDemo();

   iRun=iCheckKeyboard();
  }

 //---------------------------------------------------------// clean up

 GPUclose();                                                // close gpu window

 GPUshutdown();                                             // unload plugin

 return EXIT_SUCCESS;                                       // done
}

////////////////////////////////////////////////////////////////////////
